#----------------------------------------------------------------------------------#
#
# FILE:			__init__.py
# AUTHOR:		Peter Quinn
# DATE:			28.01.2005
# VERSION:		1.50
#
# DESCRIPTION:	RSS News reader control.
#
# INTERFACE:	set_url			W	Set the URL of the RSS provider.
#				set_browser		W	Set the browser to be used by the launcher.
#				launch			W	Launch URL within a browser.
#				interval		W	Set the interval between RSS reads.
#				refresh			RW	Display refresh trigger. (0 = Idle, 1 = Refresh)
#				read_rss		R	Reads RSS feed from provider.
#				status			R	Status of the reader. (0 = Idle, 1 = Reading)
#				get_icon		R	Get the save path of the provider icon.
#				get_items		R	Get the list of news items.
#
# STRUCTURES:	NewsItem
#
#				title				Title of current news item.
#				description			Description of current news item.
#				link				URL to the full news item.
#				date				Date news item was posted.
#				subject				Subject/Category of news item.
#				copyright			Copyright information for current news item.
#				creator				Creator of the current news item.
#
#
#----------------------------------------------------------------------------------#

from libdesklets.controls import Control, Interface, Permission
import xml.sax, RSSreader, thread, threading, re, os, dircache, commands


#----------------------------------------------------------------------------------#
#
# INewsGrab Interface
#
#
class INewsGrab(Interface):
	status			= Permission.READ
	read_rss		= Permission.READ
	get_icon		= Permission.READ
	get_items		= Permission.READ
	set_url			= Permission.WRITE
	set_browser		= Permission.WRITE
	launch			= Permission.WRITE
	interval		= Permission.WRITE
	refresh			= Permission.READWRITE


#----------------------------------------------------------------------------------#
#
# NewsGrab Class
#
#
class NewsGrab(Control, INewsGrab):

	#
	# Object initialization
	#
	def __init__(self):
		
		self._userpath	= "/var/tmp/%s" % os.environ["USER"]
		self._savepath	= "%s/newsgrab" % self._userpath
		self._parser	= xml.sax.make_parser()
		self._handler	= RSSreader.NewsHandler()
		self._browser	= "firefox"
		self._url		= ""
		self._oldurl	= ""
		self._interval	= 30 * 60 * 1000
		self._updated	= 0
		self._refresh	= 0
		self._status	= 0
		self._die		= False
		
		if not os.path.exists(self._savepath):
			
			if not os.path.exists(self._userpath):
				os.mkdir(self._userpath)
	
			os.mkdir(self._savepath)
	
		Control.__init__(self)
	
		self._add_timer(1000, self.__start)

	
	#
	# Some getter & setter methods
	#
	def __set_url(self, value):		self._url = value
	def __set_browser(self, value):	self._browser = value
	def __set_interval(self, value):self._interval = value
	def __set_refresh(self, value):	self._refresh = value
	def __get_items(self):			return self._handler.item
	def __get_refresh(self):		return self._refresh
	def __get_status(self):			return self._status


	#
	# Force a read from the RSS provider
	#
	def __read_rss(self):

		if self._status == 0:
			if self._url != self._oldurl:
				self._oldurl = self._url
				self._die = True
				self.__start()

	
	#
	# Start RSS reader thread
	#
	def __start(self):
		
		self._updated = 0
		self._refresh = 0
		self._status = 1
		self._update("status")
		
		thread.start_new_thread(self.__read_xml, ())
		self._add_timer(1000, self.__wait)

		return 0

	
	#
	# Wait for the reader to finish reading the RSS source
	#
	def __wait(self):
		
		if self._updated == 0:
			return 1
			
		elif self._updated == 1:
			self._refresh = 1
			self._update("refresh")
			self._status = 0
			self._update("status")
			
			if not self._die:
				self._add_timer(self._interval, self.__start)

			self._die = False

		return 0
	
	
	#
	# Read the XML RSS feed from the source specified by set_url()
	#
	def __read_xml(self):
		
		self._handler.item = []
		self._parser.setContentHandler(self._handler)

		try:
			self._parser.parse(self._url)
			self.__get_logo(self._handler.image[1])
	
		except:
			pass

		self._updated = 1


	#
	# Launch the URL within the browser specified by set_browser()
	#
	def __launch(self, value):
		
		p = re.compile('&')
		url = p.sub('\&', value, 0)
		
		threading.Thread(target = os.system, args = ("nice -n 0 %s %s" % (self._browser, url),)).start()
		
	
	#
	# Return the path to the current providers icon with /var/tmp
	#
	def __get_icon(self):
		
		m = re.search("http://(.*?)/", self._handler.image[1])
		
		host = m.group(1)
		path = "%s/%s" % (self._savepath, host)

		if os.path.exists(path):
			try:
				filename = "%s/%s" % (path, dircache.listdir(path)[0])
				return filename
				
			except:
				return None
			
		return None

	
	#
	# Retrieve the providers logo and save it to /var/tmp/username/newsgrab/url
	#
	def __get_logo(self, url):
		
		if url:
			m = re.search("http://(.*?)/.*/(.*)$", url)

			host		= m.group(1)
			filename	= m.group(2)
			path		= "%s/%s" % (self._savepath, host)
			filepath	= "%s/%s" % (path, filename)

			if not os.path.exists(filepath):
				if not os.path.exists(path):
					os.mkdir(path)

				try:
					try:
						img = commands.getoutput("wget -q -O - \"%s\"" % url)
						if img:
							f = open(filepath, "w")
							f.write(img)
							f.close()
	
					except:
						print "Could not save image to %s" % filepath

				except:
					print "Could not get %s" % url


	#
	# Interface
	#
	get_icon		= property(fget = __get_icon,
								doc = "Get the path of the provider logo")
	set_url			= property(fset = __set_url,
								doc = "Set the URL of where to grab the RSS from")
	set_browser		= property(fset = __set_browser,
								doc = "Set the default browser")
	launch			= property(fset = __launch,
								doc = "Launch the URL within a browser")
	interval		= property(fset = __set_interval,
								doc = "Set time between reading news from URL")
	get_items		= property(__get_items,
								doc = "Get the news items array")
	refresh			= property(fset = __set_refresh, fget = __get_refresh,
								doc = "Display refresh trigger (1 = Triggered from thread)")
	read_rss		= property(fget = __read_rss,
								doc = "Read RSS feed from provider")
	status			= property(__get_status,
								doc = "Thread Status (0 = Idle, 1 = Reading RSS)")

#----------------------------------------------------------------------------------#

def get_class(): return NewsGrab

